#include <Arduino.h>
// https://registry.platformio.org/libraries/plapointe6/EspMQTTClient
#include "EspMQTTClient.h"


#define TOPIC "your unique device id here"
#define BROKER "your mqtt broker ip here"
#define SSID "your ssid here"
#define PSK "your wifi psk here"


EspMQTTClient client(SSID, PSK, BROKER, TOPIC);

volatile bool powerStateChanged = false;

IRAM_ATTR void onPowerStateChange() {
  powerStateChanged = true;
}


void setup() {
  digitalWrite(4, LOW);
  pinMode(4, OUTPUT);
  Serial.begin(115200);  // Can only print, RX is used as sense input
  Serial.println("Setting up LWT");
  digitalWrite(3, LOW);
  pinMode(3, INPUT);
  attachInterrupt(digitalPinToInterrupt(3), onPowerStateChange, CHANGE);
  client.enableLastWillMessage("tele/" TOPIC "/LWT", "Offline");
  Serial.println("Set LWT");
}

void onBootRequest(const String& payload) {
  Serial.println("Request: " + payload);
  if (payload != "ON" && payload != "OFF" && payload != "TOGGLE") {
    return;
  }
  bool desiredState = payload == "ON";
  bool actualState = !digitalRead(3);  // GPIO3 is LOW when powered up
  if (desiredState != actualState || payload == "TOGGLE") {
    digitalWrite(4, HIGH);
    delay(100);
    digitalWrite(4, LOW);
  }
  client.publish("resp/" TOPIC "/boot", actualState ? "ON" : "OFF", true);
}

void onConnectionEstablished() {
  Serial.println("Connection established!");
  client.subscribe("cmnd/" TOPIC "/boot", onBootRequest);
  Serial.println("Subscribed to cmnd/" TOPIC "/boot");
  client.publish("tele/" TOPIC "/LWT", "Online", true);
  Serial.println("Set online!");
  powerStateChanged = true;
}


void loop() {
  client.loop();
  if (powerStateChanged) {
    powerStateChanged = false;
    bool state = !digitalRead(3); // GPIO3 is LOW when powered up
    Serial.println("Power state changed!");
    client.publish("resp/" TOPIC "/boot", state ? "ON" : "OFF", true);
  }
}
